;;; original version by merlyn
;;; LastEditDate="Mon May  1 16:04:07 1989"
;;; patch to pp-eval-expression inspired by Michael Elhadad <elhadad@cs.columbia.EDU>
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Organization: Stonehenge; netaccess via Intel, Beaverton, Oregon, USA
;; From: news@iwarp.intel.com  (Randal Schwartz)
;; To: info-gnu-emacs@prep.ai.mit.edu
;; Subject: Re: Looking for a pretty-printer
;; Date: 15 Feb 90 19:51:40 GMT
;; 
;; In article <23854@siemens.siemens.com>, mlg@acura (Michael Greenberg) writes:
;; | Does anyone have a pretty printer written in
;; | GNU Lisp?
;; 
;; Yeah, here's the one I wrote a while ago (and even posted to the net),
;; but I don't do much with it anymore cause I'm too busy doing things in
;; Perl. :-)
;; 
;; Just another Elisp hacker,
;; -- 
;; /=Randal L. Schwartz, Stonehenge Consulting Services (503)777-0095 ==========\
;; | on contract to Intel's iWarp project, Beaverton, Oregon, USA, Sol III      |
;; | merlyn@iwarp.intel.com ...!any-MX-mailer-like-uunet!iwarp.intel.com!merlyn |
;; \=Cute Quote: "Welcome to Portland, Oregon, home of the California Raisins!"=/
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun pp-to-string (object)
  "Return a string containing the pretty-printed representation of OBJECT,
any Lisp object.  Quoting characters are used when needed to make output
that `read' can handle, whenever this is possible."
  (save-excursion
    (set-buffer (generate-new-buffer " pp-to-string"))
    (unwind-protect
	(progn
	  (emacs-lisp-mode)
	  (let ((print-escape-newlines t))
	    (prin1 object (current-buffer)))
	  (goto-char (point-min))
	  (while (not (eobp))
	    ;; (message "%06d" (- (point-max) (point)))
	    (cond
	     ((looking-at "\\s(")
	      (while (looking-at "\\s(")
		(forward-char 1)))
	     ;; (looking-at "quote")
	     ((condition-case err-var
		  (prog1 t (down-list 1))
		(error nil))
	      (backward-char 1)
	      (skip-chars-backward " \t")
	      (delete-region
	       (point)
	       (progn (skip-chars-forward " \t") (point)))
	      (insert ?\n))
	     ((condition-case err-var
		  (prog1 t (up-list 1))
		(error nil))
	      (while (looking-at "\\s)")
		(forward-char 1))
	      (skip-chars-backward " \t")
	      (delete-region
	       (point)
	       (progn (skip-chars-forward " \t") (point)))
	      (insert ?\n))
	     (t (goto-char (point-max)))))
	  (goto-char (point-min))
	  (indent-sexp)
	  (buffer-string))
      (kill-buffer (current-buffer)))))

(defun pp (object &optional stream)
  "Output the pretty-printed representation of OBJECT, any Lisp object.
Quoting characters are printed when needed to make output that `read'
can handle, whenever this is possible.
Output stream is STREAM, or value of `standard-output' (which see)."
  (princ (pp-to-string object) (or stream standard-output)))

(defun pp-eval-expression (expression)
  "Evaluate EXPRESSION and pretty-print value into a new display buffer.
If the pretty-printed value fits on one line, the message line is used
instead.  Value is also consed on to front of variable  values 's
value."
  (interactive "xPp-eval: ")
  (setq values (cons (eval expression) values))
  (let ((temp-buffer-show-hook
	 (function
	  (lambda (buf)
	    (save-excursion
	      (set-buffer buf)
	      (goto-char (point-min))
	      (end-of-line 1)
	      (if (or (< (1+ (point)) (point-max))
		      (>= (- (point) (point-min)) (screen-width)))
		  (display-buffer buf)
		(message "%s" (buffer-substring (point-min) (point)))
		(kill-buffer buf)))))))
    (with-output-to-temp-buffer "*Pp Eval Output*"
      (pp (car values)))))

(defun pp-eval-last-sexp (arg)
  "Run `pp-eval-expression' on sexp before point (which see).
With argument, pretty-print output into current buffer."
  (interactive "P")
  (let ((stab (syntax-table))
	exp)
    (set-syntax-table emacs-lisp-mode-syntax-table)
    (save-excursion
      (forward-sexp -1)
      (setq exp (read (current-buffer))))
    (set-syntax-table stab)
    (if arg
	(insert (pp-to-string (eval exp)))
      (pp-eval-expression exp))))
